from dynamico.precision import zeros

def grad(mesh, s_ik):
    llm=s_ik.shape[1]
    edge_num, left, right = mesh.edge_num, mesh.left, mesh.right
    grad_ek = zeros((edge_num, llm))
    for e in range(edge_num):
        left_e = left[e]-1 # Fortran => Python indexing
        right_e = right[e]-1 # Fortran => Python indexing
        grad_ek[e,:] = s_ik[right_e,:]-s_ik[left_e,:]
    return grad_ek

def div(mesh, u_ek):
    llm=u_ek.shape[1]
    primal_num, primal_deg, primal_edge = mesh.primal_num, mesh.primal_deg, mesh.primal_edge
    primal_ne, le_de, Ai = mesh.primal_ne, mesh.le_de, mesh.Ai
    div_ik = zeros((primal_num, llm))
    for i in range(primal_num):
        div_i = 0.
        for iedge in range(primal_deg[i]):
            e = primal_edge[i,iedge]-1 # Fortran => Python indexing
            div_i = div_i + u_ek[e,:]*primal_ne[i,iedge]*le_de[e]
        div_ik[i,:] = div_i / Ai[i]
    return div_ik

def KE(mesh, u_ek):
    llm=u_ek.shape[1]
    primal_num, primal_deg, primal_edge = mesh.primal_num, mesh.primal_deg, mesh.primal_edge
    primal_ne, le_de, Ai = mesh.primal_ne, mesh.le_de, mesh.Ai
    div_ik = zeros((primal_num, llm))
    for i in range(primal_num):
        div_i = 0.
        for iedge in range(primal_deg[i]):
            e = primal_edge[i,iedge]-1 # Fortran => Python indexing
            div_i = div_i + le_de[e]*(u_ek[e,:]**2)
        div_ik[i,:] = div_i * (.25/ Ai[i])
    return div_ik

def curl(mesh, u_ek):
    llm=u_ek.shape[1]
    dual_num, dual_deg, dual_edge, dual_ne, Av = mesh.dual_num, mesh.dual_deg, mesh.dual_edge, mesh.dual_ne, mesh.Av
    curl_vk = zeros((dual_num, llm))
    for v in range(dual_num):
        curl_v = 0.
        for iedge in range(dual_deg[v]):
            e = dual_edge[v,iedge]-1 # Fortran => Python indexing
            curl_v = curl_v + u_ek[e,:]*dual_ne[v,iedge]
        curl_vk[v,:] = curl_v / Av[v]
    return curl_vk
